\name{PercolationMap}
\alias{PercolationMap}
\title{
Generate Random Category Landscape
}
\description{
A function to generate a multidimensional landscape with each cell randomly assigned a category number such that the total ratio of category types accross the landscape corresponds to a proportion vector input.
}
\usage{
PercolationMap(DimExtents, Props, Cat.Labels = 1:length(Props))
}
\arguments{
  \item{DimExtents}{
An integer vector with the length of the desired output in each dimension.  \code{length(DimExtents)} gives the number of dimensions in the output.
}
  \item{Props}{
An integer vector with a each element containing the number of cells to assign to each category type.  \code{length(Props)} gives the number of categories to assign.  Note that \code{prod(DimExtents)} must equal to or less than \code{sum(Props)} otherwise an error is displayed.  If \code{prod(DimExtents)} is less than \code{sum(Props)} then all remaining cells are assigned to an \code{NA} category.
}
  \item{Cat.Labels}{
A vector of category labels to attach to each of the assigned category types.  If \code{length(Cat.Labels)} is less than \code{length(Props)} then \code{Cat.Labels} is recycled.
}
}
\value{
A vector of length \code{prod(DimExtents)} and with a dimensional extent attribute set to \code{DimExtents}.  Each element contains the corresponding category label given in \code{Cat.Labels} to the assigned category type.
}
\references{
Gardner, R. H., Milne, B. T., Turner, M. G. and O'Neill, R. V. (1987) Neutral models for the analysis of broad-scale landscape pattern.  \emph{Landscape Ecology}, 1, 19-28.

With, K. A. (1997) The application of neutral landscape models in conservation biology.  \emph{Conservation Biology}, 11, 1069-1080.

With, K. A. and King A. W. (1997) The use and misuse of neutral landscapes models in ecology.  \emph{Oikos}, 79, 219-229.

}
\author{
Joseph Chipperfield <\email{joechip90@googlemail.com}>
}
\seealso{
\code{\link{HierarchicalCurdling}}
\code{\link{SpectralSynthesis}}
}
\examples{
# Create a 100 x 100 landscape with 3000 "grass" cells, 2000 "sand" cells, and
# 5000 "bog" cells
dims.in <- rep(100,2)				# Dimension extent
props <- c(3000, 2000, 5000)			# Proportions of each habitat type
cat.labels <- c("grass", "sand", "bog")		# Category labels
output.map <- PercolationMap(dims.in, props, cat.labels)
# Code the outputs into values for display
coded.map <- ifelse(output.map == "grass", 1, ifelse(output.map == "sand", 2, 3))
image(coded.map, col = c("green", "yellow", "brown"))
}
\keyword{models}
