#include "stdafx.h"
#include "Exception.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{
	// -----------------------------
	//  FUNCTION TO RAISE EXCEPTION
	// -----------------------------
	// Function that raises an exception
	void RaiseException(const char *pMessage, size_t ulLine, const char *pRoutine, const char *pFile, ErrorClass cErrorType) throw(Exception)
	{
		Exception cError;
		cError.Initialise(pMessage, pRoutine, ulLine, pFile, cErrorType);
		throw(cError);
	}

	// --------------------------------
	//  METHODS OF THE EXCEPTION CLASS
	// --------------------------------
	// Empty initialisation function
	void Exception::Initialise() throw()
	{
		ulLine = 0;
		pRoutine[0] = pFile[0] = pMessage[0] = '\0';
		cErrorType = e_UnknownError;
	}

	// Parameterised initialisation function
	void Exception::Initialise(const char* pMessageIn, const char *pRoutineIn, size_t ulLineIn, const char *pFileIn, ErrorClass cTypeIn) throw()
	{
		Initialise();
		ulLine = ulLineIn;
		strncpy(pRoutine, pRoutineIn, EXCEPTION_BUFFSIZE + 1);
		strncpy(pMessage, pMessageIn, EXCEPTION_BUFFSIZE + 1);
		strncpy(pFile, pFileIn, EXCEPTION_BUFFSIZE + 1);
		cErrorType = cTypeIn;
	}

	// Copy initialisation function
	void Exception::Initialise(const Exception &cObIn) throw()
	{
		if(&cObIn != this)
		{
			Initialise();
			ulLine = cObIn.ulLine;
			strncpy(pRoutine, cObIn.pRoutine, EXCEPTION_BUFFSIZE + 1);
			strncpy(pMessage, cObIn.pMessage, EXCEPTION_BUFFSIZE + 1);
			strncpy(pFile, cObIn.pFile, EXCEPTION_BUFFSIZE + 1);
			cErrorType = cObIn.cErrorType;
		}
	}

	// Function to return the error message associated with an exception
	char *Exception::GetErrorMessage(char *pStrOut, size_t ulSizeOut) const throw()
	{
		size_t ulMinSize = ulSizeOut < EXCEPTION_BUFFSIZE ? ulSizeOut : EXCEPTION_BUFFSIZE;
		if(ulMinSize > 0)
		{
			strncpy(pStrOut, pMessage, ulMinSize);
			pStrOut[ulMinSize - 1] = '\0';
		}
		return pStrOut;
	}

	// Function to return the routine from which an exception was thrown
	char *Exception::GetErrorRoutine(char *pStrOut, size_t ulSizeOut) const throw()
	{
		size_t ulMinSize = ulSizeOut < EXCEPTION_BUFFSIZE ? ulSizeOut : EXCEPTION_BUFFSIZE;
		if(ulMinSize > 0)
		{
			strncpy(pStrOut, pRoutine, ulMinSize);
			pStrOut[ulMinSize - 1] = '\0';
		}
		return pStrOut;
	}

	// Function to return the source file from which an exception was thrown
	char *Exception::GetErrorFile(char *pStrOut, size_t ulSizeOut) const throw()
	{
		size_t ulMinSize = ulSizeOut < EXCEPTION_BUFFSIZE ? ulSizeOut : EXCEPTION_BUFFSIZE;
		if(ulMinSize > 0)
		{
			strncpy(pStrOut, pFile, ulMinSize);
			pStrOut[ulMinSize - 1] = '\0';
		}
		return pStrOut;
	}

	// Test of equality of error messages
	bool Exception::Equal(const Exception &cObIn) const throw()
	{
		return strncmp(pMessage, cObIn.pMessage, EXCEPTION_BUFFSIZE) == 0 &&
			strncmp(pRoutine, cObIn.pRoutine, EXCEPTION_BUFFSIZE) == 0 &&
			strncmp(pFile, cObIn.pFile, EXCEPTION_BUFFSIZE) == 0 &&
			ulLine == cObIn.ulLine && cErrorType == cObIn.cErrorType ? true : false;
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
