#include "stdafx.h"
#include "ExtString.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{	// -----------------------------------
	//  METHODS FOR EXTENDED STRING CLASS
	// -----------------------------------

	// Null initialisation function
	void ExtString::Initialise() throw()
	{
		if(pString != NULL)
		{
			delete [] pString;
			pString = NULL;
		}
		ulStringSize = 0;
	}

	// Parameterised initialisation function 1
	void ExtString::Initialise(const char *pStringIn, size_t ulSizeIn) throw(Exception)
	{
		size_t ulStrLen = 0;	// Variable to store string length
		Initialise();
		if(pStringIn != NULL)
		{
			if(*pStringIn == '\0' || ulSizeIn == 0)
			{	// Special situation where string given is ""
				pString = AllocateMemory<char>(1);
				*pString = '\0';
			}
			else
			{	// Otherwise allocate memory for string and copy it across
				ulStrLen = strlen(pStringIn);
				ulStrLen = ulStrLen < ulSizeIn ? ulStrLen : ulSizeIn;
				pString = AllocateMemory<char>(ulStrLen + 1);
				ulStringSize = ulStrLen;
				strncpy(pString, pStringIn, ulStrLen + 1);
			}
		}
	}

	// Parameterised initialisation function 2
	void ExtString::Initialise(const char *pStringIn) throw(Exception)
	{
		size_t ulSizeIn = 0;
		Initialise();
		if(pStringIn != NULL)
		{
			if(*pStringIn == '\0')
			{	// Special situation where string given is ""
				pString = AllocateMemory<char>(1);
				*pString = '\0';
			}
			else
			{	// Otherwise allocate memory for string and copy it across
				ulSizeIn = strlen(pStringIn);
				pString = AllocateMemory<char>(ulSizeIn + 1);
				ulStringSize = ulSizeIn;
				strncpy(pString, pStringIn, ulSizeIn + 1);
			}
		}
	}

	// Copy initialisation function
	void ExtString::Initialise(const ExtString &cObIn) throw(Exception)
	{	// Check to ensure input object is not the same as the calling object
		if(&cObIn != this)
		{
			Initialise(cObIn.pString, cObIn.ulStringSize);
		}
	}
	
	// Special value initialisation functions
	void ExtString::Initialise(SpecialValues eSpecIn) throw(Exception)
	{
		Initialise();
		if(eSpecIn != e_MissingValue)
		{
			RaiseException("invalid special value for the extended string class", __LINE__, "StdExtensions::ExtString::Initialise(SpecialValues)", __FILE__, e_DomainError);
		}
	}

	// Function to test equality of two strings 1
	ExtBool ExtString::Equal(const ExtString &cObIn) const throw()
	{
		ExtBool cOut;
		size_t ulMax = 0;
		if(pString == NULL || cObIn.pString == NULL)
		{	// Either string is a missing value and so a missing value boolean is returned
			cOut.Initialise(e_MissingValue);
		}
		else
		{
			if(ulStringSize == 0 && cObIn.ulStringSize == 0)
			{	// Both strings are empty - return true
				cOut.Initialise(true);
			}
			else
			{	// Otherwise compare the two strings
				ulMax = ulStringSize > cObIn.ulStringSize ? ulStringSize : cObIn.ulStringSize;
				cOut.Initialise(strncmp(pString, cObIn.pString, ulMax) == 0);
			}
		}
		return cOut;
	}

	// Function to test equality of two strings 2
	ExtBool ExtString::Equal(const char *pStringIn) const throw()
	{
		ExtBool cOut;
		size_t ulMax = 0;
		if(pString == NULL || pStringIn == NULL)
		{	// Either string is a missing value and so a missing value boolean is returned
			cOut.Initialise(e_MissingValue);
		}
		else
		{
			ulMax = strlen(pStringIn);
			if(ulStringSize == 0 && ulMax == 0)
			{	// Both strings are empty - return true
				cOut.Initialise(true);
			}
			else
			{	// Otherwise compare the two strings
				ulMax = ulStringSize > ulMax ? ulStringSize : ulMax;
				cOut.Initialise(strncmp(pString, pStringIn, ulMax) == 0);
			}
		}
		return cOut;
	}

	// Returns the string
	char *ExtString::RetrieveCString(char *pStringOut, size_t ulSize) const throw()
	{
		size_t ulMin = 0;
		ulMin = ulSize < ulStringSize + 1 ? ulSize : ulStringSize + 1;
		if(pStringOut != NULL || ulSize != 0)
		{	// If the outputs are valid then export the string
			*pStringOut = '\0';
			if(pString == NULL || ulStringSize == 0)
			{	// Copy an empty string to the output vector
				*pStringOut = '\0';
			}
			else
			{	// Copy the string to the output vector
				strncpy(pStringOut, pString, ulMin);
				pStringOut[ulMin - 1] = '\0';
			}
		}
		// Return the string output
		return pStringOut;
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
