#include "stdafx.h"
#include "MathFuncs.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{
	namespace
	{
		// ------------------------------------------------
		//  STATIC FUNCTIONS AND VARIABLES CALLED FROM FFT
		// ------------------------------------------------

		// Define constants used in fft
		const double dSin60 = 0.86602540378443865;	// sin(60 degrees)
		const double dCos72 = 0.30901699437494742;	// cos(72 degrees)
		const double dSin72 = 0.95105651629515357;	// sin(72 degrees)
		const size_t ulNFactor = 11;				// Number of factors

		// Define constants used in erf
		// Mathematical constants
		const double dFour = 4.0;
		const double dOne = 1.0;
		const double dHalf = 0.5;
		const double dTwo = 2.0;
		const double dZero = 0.0;
		const double dSqrPi = 5.6418958354775628695e-01;
		const double dThresh = 0.46875;
		const double dSixTen = 16.0;
		// Machine-dependent constants
		const double dXInf = 1.79e308;
		const double dXNeg = -26.628;
		const double dXSmall = 1.11e-16;
		const double dXBig = 26.543;
		const double dXHuge = 6.71e07;
		const double dXMax = 2.53e307;
		// Coefficients for approximation to erf in first interval
		const double pA[5] = {
			3.16112374387056560e00,		1.13864154151050156e02,
			3.77485237685302021e02,		3.20937758913846947e03,
			1.85777706184603153e-01
		};
		const double pB[4] = {
			2.36012909523441209e01,		2.44024637934444173e02,
			1.28261652607737228e03,		2.84423683343917062e03
		};
		// Coefficients for approximation to erfc in second interval
		const double pC[9] = {
			5.64188496988670089e-01,	8.88314979438837594e00,
			6.61191906371416295e01,		2.98635138197400131e02,
			8.81952221241769090e02,		1.71204761263407058e03,
			2.05107837782607147e03,		1.23033935479799725e03,
			2.15311535474403846e-08
		};
		const double pD[8] = {
			1.57449261107098347e01,		1.17693950891312499e02,
			5.37181101862009858e02,		1.62138957456669019e03,
			3.29079923573345963e03,		4.36261909014324716e03,
			3.43936767414372164e03,		1.23033935480374942e03
		};
		// Coefficients for approximation to erfc in third interval
		const double pP[6] = {
			3.05326634961232344e-01,	3.60344899949804439e-01,
			1.25781726111229246e-01,	1.60837851487422766e-02,
			6.58749161529837803e-04,	1.63153871373020978e-02
		};
		const double pQ[5] = {
			2.56852019228982242e00,		1.87295284992346047e00,
			5.27905102951428412e-01,	6.05183413124413191e-02,
			2.33520497626869185e-03
		};

		// Function to calculate the error function or complimentary error function
		double CalErf(double dArg, int iJInt) throw()
		{
			double dDel, dResult, dX, dXNum, dY, dYSq, dXDen;
			int iI;
			bool bGoto = false;
			dDel = dResult = dX = dXNum = dY = dYSq = dXDen = 0.0;
			iI = 0;
			dX = dArg;
			dY = fabs(dX);
			// Evaluate erf for |x| <= 0.46875
			if(dY <= dThresh)
			{
				dYSq = dZero;
				if(dXSmall < dY)
				{
					dYSq = dY * dY;
				}
				dXNum = pA[4] * dYSq;
				dXDen = dYSq;
				for(iI = 0; iI < 3; ++iI)
				{
					dXNum = (dXNum + pA[size_t(iI)]) * dYSq;
					dXDen = (dXDen + pB[size_t(iI)]) * dYSq;
				}
				dResult = dX * (dXNum + pA[3]) / (dXDen + pB[3]);
				if(iJInt != 0)
				{
					dResult = dOne - dResult;
				}
				if(iJInt == 2)
				{
					dResult = exp(dYSq) * dResult;
				}
				return dResult;
			}
			// Evaluate erfc for 0.46875 <= |x| <= 4.0
			else if(dY <= dFour)
			{
				dXNum = pC[8] * dY;
				dXDen = dY;
				for(iI = 0; iI < 7; ++iI)
				{
					dXNum = (dXNum + pC[size_t(iI)]) * dY;
					dXDen = (dXDen + pD[size_t(iI)]) * dY;
				}
				dResult = (dXNum + pC[7]) / (dXDen + pD[7]);
				if(iJInt != 2)
				{
					dYSq = Trunc(dY * dSixTen) / dSixTen;
					dDel = (dY - dYSq) * (dY + dYSq);
					dResult = exp(-dYSq * dYSq) * exp(-dDel) * dResult;
				}
			}
			// Evaluate erfc for 4.0 < |x|
			else
			{
				dResult = dZero;
				if(dXBig <= dY)
				{
					if(iJInt != 2 || dXMax <= dY)
					{
						bGoto = true;
					}
					if(dXHuge <= dY)
					{
						dResult = dSqrPi / dY;
						bGoto = true;
					}
				}
				if(!bGoto)
				{
					dYSq = dOne / (dY * dY);
					dXNum = pP[5] * dYSq;
					dXDen = dYSq;
					for(iI = 0; iI < 4; ++iI)
					{
						dXNum = (dXNum + pP[size_t(iI)]) * dYSq;
						dXDen = (dXDen + pQ[size_t(iI)]) * dYSq;
					}
					dResult = dYSq * (dXNum + pP[4]) / (dXDen + pQ[4]);
					dResult = (dSqrPi - dResult) / dY;
					if(iJInt != 2)
					{
						dYSq = Trunc(dY * dSixTen) / dSixTen;
						dDel = (dY - dYSq) * (dY + dYSq);
						dResult = exp(-dYSq * dYSq) * exp(-dDel) * dResult;
					}
				}
			}
			// Fix up for negative argument, erf, etc.
			if(iJInt == 0)
			{
				dResult = (dHalf - dResult) + dHalf;
				if(dX < dZero)
				{
					dResult = -dResult;
				}
			}
			else if(iJInt == 1)
			{
				if(dX < dZero)
				{
					dResult = dTwo - dResult;
				}
			}
			else
			{
				if(dX < dZero)
				{
					if(dX < dXNeg)
					{
						dResult = dXInf;
					}
					else
					{
						dYSq = Trunc(dX * dSixTen) / dSixTen;
						dDel = (dX - dYSq) * (dX + dYSq);
						dY = exp(dYSq * dYSq) * exp(dDel);
						dResult = (dY + dY) - dResult;
					}
				}
			}
			return dResult;
		}

		// Function to delete fft vectors
		void fft_free(size_t &ulSpaceAlloced, size_t &ulMaxPermAlloced, double **pTmp, int *&pPerm) throw()
		{
			ulSpaceAlloced = ulMaxPermAlloced = 0;
			if(pTmp != NULL)
			{	// Delete the temporary storage arrays
				if(pTmp[0] != NULL)
				{
					delete [] pTmp[0];
				}
				if(pTmp[1] != NULL)
				{
					delete [] pTmp[1];
				}
				if(pTmp[2] != NULL)
				{
					delete [] pTmp[2];
				}
				if(pTmp[3] != NULL)
				{
					delete [] pTmp[3];
				}
				pTmp[0] = pTmp[1] = pTmp[2] = pTmp[3] = NULL;
			}
			if(pPerm != NULL)
			{	// Delete the permutation vector
				delete [] pPerm;
				pPerm = NULL;
			}
		}

		// Small function to extract factors
		int& fft_factor(int *pFactor, size_t ulElement) throw(StdExtensions::Exception)
		{	// Raise exception if factor is requested outside of array range
			if(ulElement >= ulNFactor)
			{
				RaiseException("number of factors exceeded", __LINE__, "StdExtensions::fft_factor(int *, size_t)", __FILE__, e_DomainError);
			}
			return pFactor[ulElement];
		}

		// Singleton's mixed radix routine
		int fft_radix(double *pRe, double *pIm, size_t ulNTotal, size_t ulNPass, size_t ulNSpan, int iSign, int iMaxFactors,
			int iMaxPerm, size_t &ulSpaceAlloced, size_t &ulMaxPermAlloced, double **pTmp, int *&pPerm, int *&pFactor) throw(StdExtensions::Exception)
		{
			size_t ulMin, ulIter;
			int iI, iMFactor, iKSpan, iISpan, iInc;
			int iJ, iJC, iJF, iJJ, iK, iK1, iK2, iK3, iK4, iKK, iKT, iNN, iNS, iNT;
			int *pPermTmp;
			double dRadF;
			double dC1, dC2, dC3, dCD, dAA, dAJ, dAK, dAJM, dAJP, dAKM, dAKP;
			double dS1, dS2, dS3, dSD, dBB, dBJ, dBK, dBJM, dBJP, dBKM, dBKP;
			double *pRTmp, *pITmp, *pCos, *pSin;
			double dS60, dC72, dS72, dPI2;
			double *pTmpTmp[4];
			bool bGotoPermuteResultsLabel, bGotoPermuteMultiLabel, bGotoPermuteSingleLabel, bLoopBreak;
			bGotoPermuteResultsLabel = bGotoPermuteMultiLabel = bGotoPermuteSingleLabel = false;
			bLoopBreak = false;
			// Initialise variables to keep GCC happy
			ulMin = ulIter = 0;
			iI = iMFactor = iKSpan = iISpan = iInc = 0;
			iJ = iJC = iJF = iJJ = iK = iK1 = iK2 = iK3 = iK4 = iKK = iKT = iNN = iNS = iNT = 0;
			pPermTmp = NULL;
			dRadF = 0.0;
			dC1 = dC2 = dC3 = dCD = dAA = dAJ = dAK = dAJM = dAJP = dAKM = dAKP = 0.0;
			dS1 = dS2 = dS3 = dSD = dBB = dBJ = dBK = dBJM = dBJP = dBKM = dBKP = 0.0;
			pRTmp = pITmp = pCos = pSin = NULL;
			dS60 = dSin60;	// sin(60 degrees)
			dC72 = dCos72;	// cos(72 degrees)
			dS72 = dSin72;	// sin(72 degrees)
			dPI2 = EXT_PI;	// Use PI first, 2*PI later
			pTmpTmp[0] = pTmpTmp[1] = pTmpTmp[2] = pTmpTmp[3] = NULL;
			// Parameter adhustments, was fortran so fix zero-offset
			--pRe;
			--pIm;
			if(ulNPass >= 2)
			{	// Allocate storage
				if(ulSpaceAlloced < size_t(iMaxFactors) * sizeof(double))
				{
					if(ulSpaceAlloced == 0)
					{	// First time
						ulSpaceAlloced = size_t(iMaxFactors) * sizeof(double);
						pTmp[0] = AllocateMemory<double>(size_t(iMaxFactors));
						pTmp[1] = AllocateMemory<double>(size_t(iMaxFactors));
						pTmp[2] = AllocateMemory<double>(size_t(iMaxFactors));
						pTmp[3] = AllocateMemory<double>(size_t(iMaxFactors));
					}
					else
					{	// Done this before - Reallocate memory
						pTmpTmp[0] = AllocateMemory<double>(size_t(iMaxFactors));
						pTmpTmp[1] = AllocateMemory<double>(size_t(iMaxFactors));
						pTmpTmp[2] = AllocateMemory<double>(size_t(iMaxFactors));
						pTmpTmp[3] = AllocateMemory<double>(size_t(iMaxFactors));
						ulMin = ulSpaceAlloced / sizeof(double);
						// Copy the contents of the old arrays across
						for(ulIter = 0; ulIter < ulMin; ++ulIter)
						{
							pTmpTmp[0][ulIter] = pTmp[0][ulIter];
							pTmpTmp[1][ulIter] = pTmp[1][ulIter];
							pTmpTmp[2][ulIter] = pTmp[2][ulIter];
							pTmpTmp[3][ulIter] = pTmp[3][ulIter];
						}
						// Delete the old arrays
						delete [] pTmp[0];
						delete [] pTmp[1];
						delete [] pTmp[2];
						delete [] pTmp[3];
						// Copy the new arrays to the position of the old arrays
						pTmp[0] = pTmpTmp[0];
						pTmp[1] = pTmpTmp[1];
						pTmp[2] = pTmpTmp[2];
						pTmp[3] = pTmpTmp[3];
						// Reset the new arrays
						pTmpTmp[0] = pTmpTmp[1] = pTmpTmp[2] = pTmpTmp[3] = NULL;
						ulSpaceAlloced = size_t(iMaxFactors) * sizeof(double);
					}
				}
				else
				{	// Allow full use of alloc'd space
					iMaxFactors = int(ulSpaceAlloced / sizeof(double));
				}
				if(ulMaxPermAlloced < size_t(iMaxPerm))
				{
					if(ulMaxPermAlloced == 0)
					{	// First time
						pPerm = AllocateMemory<int>(size_t(iMaxPerm));
					}
					else
					{	// Done this before
						// Allocate storage for temporary array before copying
						// contents across
						pPermTmp = AllocateMemory<int>(size_t(iMaxPerm));
						for(ulIter = 0; ulIter < ulMaxPermAlloced; ++ulIter)
						{
							pPermTmp[ulIter] = pPerm[ulIter];
						}
						// Delete the old array
						delete [] pPerm;
						// Set the new array
						pPerm = pPermTmp;
						pPermTmp = NULL;
					}
					ulMaxPermAlloced = size_t(iMaxPerm);
				}
				else
				{	// Allow full use of alloc'd space
					iMaxPerm = int(ulMaxPermAlloced);
				}
				if(pTmp[0] == NULL || pTmp[1] == NULL || pTmp[2] == NULL || pTmp[3] == NULL || pPerm == NULL)
				{	// Errors in memory allocation
					RaiseException("memory error thrown during call of Singleton's mixed radix routine", __LINE__, "fft_radix(double *, double *, size_t, size_t, size_t, int, int, int, size_t, size_t, double **, int *&, int *&)", __FILE__, e_HeapMemoryLow);
				}
				else
				{	// No memory errors
					pRTmp = pTmp[0];
					pITmp = pTmp[1];
					pCos = pTmp[2];
					pSin = pTmp[3];
					// Function body
					iInc = iSign;
					if(iSign < 0)
					{	// Absolute value
						dS72 = -dS72;
						dS60 = -dS60;
						dPI2 = -dPI2;
						iInc = -iInc;
					}
					// Adjust for strange increments
					iNT = iInc * int(ulNTotal);
					iNS = iInc * int(ulNSpan);
					iKSpan  = iNS;
					iNN = iNT - iInc;
					iJC = iNS / int(ulNPass);
					dRadF = dPI2 * double(iJC);
					dPI2 *= 2.0;		// use 2*PI from here on
					iI = 0;
					iJF = 0;
					// Determine the factors of n
					iMFactor = 0;
					iK = int(ulNPass);
					while(iK % 16 == 0)
					{
						iMFactor++;
						fft_factor(pFactor, size_t(iMFactor - 1)) = 4;
						iK /= 16;
					}
					iJ = 3;
					iJJ = 9;
					do
					{
						while(iK % iJJ == 0)
						{
							iMFactor++;
							fft_factor(pFactor, size_t(iMFactor - 1)) = iJ;
							iK /= iJJ;
						}
						iJ += 2;
						iJJ = iJ * iJ;
					} while(iJJ <= iK);
					if(iK <= 4)
					{
						iKT = iMFactor;
						fft_factor(pFactor, size_t(iMFactor)) = iK;
						if(iK != 1)
						{
							iMFactor++;
						}
					}
					else
					{
						if(iK - (iK / 4 << 2) == 0)
						{
							iMFactor++;
							fft_factor(pFactor, size_t(iMFactor - 1)) = 2;
							iK /= 4;
						}
						iKT = iMFactor;
						iJ = 2;
						do
						{
							if(iK % iJ == 0)
							{
								iMFactor++;
								fft_factor(pFactor, size_t(iMFactor - 1)) = iJ;
								iK /= iJ;
							}
							iJ = ((iJ + 1) / 2 << 1) + 1;
						} while(iJ <= iK);
					}
					if(iKT != 0)
					{
						iJ = iKT;
						do
						{
							iMFactor++;
							fft_factor(pFactor, size_t(iMFactor - 1)) = fft_factor(pFactor, size_t(iJ - 1));
							iJ--;
						} while(iJ != 0);
					}
					// Test that iMFactors is in range
					if(iMFactor > int(ulNFactor))
					{	// Errors in memory allocation
						RaiseException("number of factors exceeded", __LINE__, "fft_radix(double *, double *, size_t, size_t, size_t, int, int, int, size_t, size_t, double **, int *&, int *&)", __FILE__, e_DomainError);
					}
					// Compute Fourier transform
					do
					{
						bGotoPermuteResultsLabel = false;
						dSD = dRadF / double(iKSpan);
						dCD = sin(dSD);
						dCD = 2.0 * dCD * dCD;
						dSD = sin(dSD + dSD);
						iKK = 1;
						iI++;
						switch(fft_factor(pFactor, size_t(iI - 1)))
						{
						case 2:	// Transform for factor of 2 (including rotation factor)
							iKSpan /= 2;
							iK1 = iKSpan + 2;
							do
							{
								do
								{
									iK2 = iKK + iKSpan;
									dAK = pRe[size_t(iK2)];
									dBK = pIm[size_t(iK2)];
									pRe[size_t(iK2)] = pRe[size_t(iKK)] - dAK;
									pIm[size_t(iK2)] = pIm[size_t(iKK)] - dBK;
									pRe[size_t(iKK)] += dAK;
									pIm[size_t(iKK)] += dBK;
									iKK = iK2 + iKSpan;
								} while(iKK <= iNN);
								iKK -= iNN;
							} while(iKK <= iJC);
							if(iKK > iKSpan)
							{	// Exit infinite loop
								bGotoPermuteResultsLabel = true;
							}
							else
							{
								do
								{
									dC1 = 1.0 - dCD;
									dS1 = dSD;
									do
									{
										do
										{
											do
											{
												iK2 = iKK + iKSpan;
												dAK = pRe[size_t(iKK)] - pRe[size_t(iK2)];
												dBK = pIm[size_t(iKK)] - pIm[size_t(iK2)];
												pRe[size_t(iKK)] += pRe[size_t(iK2)];
												pIm[size_t(iKK)] += pIm[size_t(iK2)];
												pRe[size_t(iK2)] = dC1 * dAK - dS1 * dBK;
												pIm[size_t(iK2)] = dS1 * dAK + dC1 * dBK;
												iKK = iK2 + iKSpan;
											} while(iKK < iNT);
											iK2 = iKK - iNT;
											dC1 = -dC1;
											iKK = iK1 - iK2;
										} while(iKK > iK2);
										dAK = dC1 - (dCD * dC1 + dSD * dS1);
										dS1 = dSD * dC1 - dCD * dS1 + dS1;
										dC1 = 2.0 - (dAK * dAK + dS1 * dS1);
										dS1 *= dC1;
										dC1 *= dAK;
										iKK += iJC;
									} while(iKK < iK2);
									iK1 += iInc + iInc;
									iKK = (iK1 - iKSpan) / 2 + iJC;
								} while(iKK <= iJC + iJC);
							}
							break;
						case 4:	// Transform for factor of 4
							iISpan = iKSpan;
							iKSpan /= 4;
							do
							{
								dC1 = 1.0;
								dS1 = 0.0;
								do
								{
									do
									{
										iK1 = iKK + iKSpan;
										iK2 = iK1 + iKSpan;
										iK3 = iK2 + iKSpan;
										dAKP = pRe[size_t(iKK)] + pRe[size_t(iK2)];
										dAKM = pRe[size_t(iKK)] - pRe[size_t(iK2)];
										dAJP = pRe[size_t(iK1)] + pRe[size_t(iK3)];
										dAJM = pRe[size_t(iK1)] - pRe[size_t(iK3)];
										dBKP = pIm[size_t(iKK)] + pIm[size_t(iK2)];
										dBKM = pIm[size_t(iKK)] - pIm[size_t(iK2)];
										dBJP = pIm[size_t(iK1)] + pIm[size_t(iK3)];
										dBJM = pIm[size_t(iK1)] - pIm[size_t(iK3)];
										pRe[size_t(iKK)] = dAKP + dAJP;
										pIm[size_t(iKK)] = dBKP + dBJP;
										dAJP = dAKP - dAJP;
										dBJP = dBKP - dBJP;
										if(iSign < 0)
										{
											dAKP = dAKM + dBJM;
											dBKP = dBKM - dAJM;
											dAKM -= dBJM;
											dBKM += dAJM;
										}
										else
										{
											dAKP = dAKM - dBJM;
											dBKP = dBKM + dAJM;
											dAKM += dBJM;
											dBKM -= dAJM;
										}
										// Avoid useless multiplies
										if(dS1 == 0.0)
										{
											pRe[size_t(iK1)] = dAKP;
											pRe[size_t(iK2)] = dAJP;
											pRe[size_t(iK3)] = dAKM;
											pIm[size_t(iK1)] = dBKP;
											pIm[size_t(iK2)] = dBJP;
											pIm[size_t(iK3)] = dBKM;
										}
										else
										{
											pRe[size_t(iK1)] = dAKP * dC1 - dBKP * dS1;
											pRe[size_t(iK2)] = dAJP * dC2 - dBJP * dS2;
											pRe[size_t(iK3)] = dAKM * dC3 - dBKM * dS3;
											pIm[size_t(iK1)] = dAKP * dS1 + dBKP * dC1;
											pIm[size_t(iK2)] = dAJP * dS2 + dBJP * dC2;
											pIm[size_t(iK3)] = dAKM * dS3 + dBKM * dC3;
										}
										iKK = iK3 + iKSpan;
									} while(iKK <= iNT);
									dC2 = dC1 - (dCD * dC1 + dSD * dS1);
									dS1 = dSD * dC1 - dCD * dS1 + dS1;
									dC1 = 2.0 - (dC2 * dC2 + dS1 * dS1);
									dS1 *= dC1;
									dC1 *= dC2;
									// values of dC2, dC3, dS2, dS3 that will get used next time
									dC2 = dC1 * dC1 - dS1 * dS1;
									dS2 = 2.0 * dC1 * dS1;
									dC3 = dC2 * dC1 - dS2 * dS1;
									dS3 = dC2 * dS1 + dS2 * dC1;
									iKK = iKK - iNT + iJC;
								} while(iKK <= iKSpan);
								iKK = iKK - iKSpan + iInc;
							} while(iKK <= iJC);
							if(iKSpan == iJC)
							{
								bGotoPermuteResultsLabel = true;
							}
							break;
						default:// Transform for odd factors
							iK = fft_factor(pFactor, size_t(iI - 1));
							iISpan = iKSpan;
							iKSpan /= iK;
							switch(iK)
							{
							case 3:	// Transform for factor of 3 (optional code)
								do
								{
									do
									{
										iK1 = iKK + iKSpan;
										iK2 = iK1 + iKSpan;
										dAK = pRe[size_t(iKK)];
										dBK = pIm[size_t(iKK)];
										dAJ = pRe[size_t(iK1)] + pRe[size_t(iK2)];
										dBJ = pIm[size_t(iK1)] + pIm[size_t(iK2)];
										pRe[size_t(iKK)] = dAK + dAJ;
										pIm[size_t(iKK)] = dBK + dBJ;
										dAK -= 0.5 * dAJ;
										dBK -= 0.5 * dBJ;
										dAJ = (pRe[size_t(iK1)] - pRe[size_t(iK2)]) * dS60;
										dBJ = (pIm[size_t(iK1)] - pIm[size_t(iK2)]) * dS60;
										pRe[size_t(iK1)] = dAK - dBJ;
										pRe[size_t(iK2)] = dAK + dBJ;
										pIm[size_t(iK1)] = dBK + dAJ;
										pIm[size_t(iK2)] = dBK - dAJ;
										iKK = iK2 + iKSpan;
									} while(iKK < iNN);
									iKK -= iNN;
								} while(iKK <= iKSpan);
								break;
							case 5:	// transform for factor of 5 (optional code)
								dC2 = dC72 * dC72 - dS72 * dS72;
								dS2 = 2.0 * dC72 * dS72;
								do
								{
									do
									{
										iK1 = iKK + iKSpan;
										iK2 = iK1 + iKSpan;
										iK3 = iK2 + iKSpan;
										iK4 = iK3 + iKSpan;
										dAKP = pRe[size_t(iK1)] + pRe[size_t(iK4)];
										dAKM = pRe[size_t(iK1)] - pRe[size_t(iK4)];
										dBKP = pIm[size_t(iK1)] + pIm[size_t(iK4)];
										dBKM = pIm[size_t(iK1)] - pIm[size_t(iK4)];
										dAJP = pRe[size_t(iK2)] + pRe[size_t(iK3)];
										dAJM = pRe[size_t(iK2)] - pRe[size_t(iK3)];
										dBJP = pIm[size_t(iK2)] + pIm[size_t(iK3)];
										dBJM = pIm[size_t(iK2)] - pIm[size_t(iK3)];
										dAA = pRe[size_t(iKK)];
										dBB = pIm[size_t(iKK)];
										pRe[size_t(iKK)] = dAA + dAKP + dAJP;
										pIm[size_t(iKK)] = dBB + dBKP + dBJP;
										dAK = dAKP * dC72 + dAJP * dC2 + dAA;
										dBK = dBKP * dC72 + dBJP * dC2 + dBB;
										dAJ = dAKM * dS72 + dAJM * dS2;
										dBJ = dBKM * dS72 + dBJM * dS2;
										pRe[size_t(iK1)] = dAK - dBJ;
										pRe[size_t(iK4)] = dAK + dBJ;
										pIm[size_t(iK1)] = dBK + dAJ;
										pIm[size_t(iK4)] = dBK - dAJ;
										dAK = dAKP * dC2 + dAJP * dC72 + dAA;
										dBK = dBKP * dC2 + dBJP * dC72 + dBB;
										dAJ = dAKM * dS2 - dAJM * dS72;
										dBJ = dBKM * dS2 - dBJM * dS72;
										pRe[size_t(iK2)] = dAK - dBJ;
										pRe[size_t(iK3)] = dAK + dBJ;
										pIm[size_t(iK2)] = dBK + dAJ;
										pIm[size_t(iK3)] = dBK - dAJ;
										iKK = iK4 + iKSpan;
									} while(iKK < iNN);
									iKK -= iNN;
								} while(iKK <= iKSpan);
								break;
							default:
								if(iK != iJF)
								{
									iJF = iK;
									dS1 = dPI2 / double(iK);
									dC1 = cos(dS1);
									dS1 = sin(dS1);
									if(iJF > iMaxFactors)
									{	// Errors in memory allocation
										RaiseException("number of factors exceeded", __LINE__, "fft_radix(double *, double *, size_t, size_t, size_t, int, int, int, size_t, size_t, double **, int *&, int *&)", __FILE__, e_DomainError);
									}
									pCos[size_t(iJF - 1)] = 1.0;
									pSin[size_t(iJF - 1)] = 0.0;
									iJ = 1;
									do
									{
										pCos[size_t(iJ - 1)] = pCos[size_t(iK - 1)] * dC1 + pSin[size_t(iK - 1)] * dS1;
										pSin[size_t(iJ - 1)] = pCos[size_t(iK - 1)] * dS1 - pSin[size_t(iK - 1)] * dC1;
										iK--;
										pCos[size_t(iK - 1)] = pCos[size_t(iJ - 1)];
										pSin[size_t(iK - 1)] = -pSin[size_t(iJ - 1)];
										iJ++;
									} while(iJ < iK);
								}
								do
								{
									do
									{
										iK1 = iKK;
										iK2 = iKK + iISpan;
										dAK = dAA = pRe[size_t(iKK)];
										dBK = dBB = pIm[size_t(iKK)];
										iJ = 1;
										iK1 += iKSpan;
										do
										{
											iK2 -= iKSpan;
											iJ++;
											pRTmp[size_t(iJ - 1)] = pRe[size_t(iK1)] + pRe[size_t(iK2)];
											dAK += pRTmp[size_t(iJ - 1)];
											pITmp[size_t(iJ - 1)] = pIm[size_t(iK1)] + pIm[size_t(iK2)];
											dBK += pITmp[size_t(iJ - 1)];
											iJ++;
											pRTmp[size_t(iJ - 1)] = pRe[size_t(iK1)] - pRe[size_t(iK2)];
											pITmp[size_t(iJ - 1)] = pIm[size_t(iK1)] - pIm[size_t(iK2)];
											iK1 += iKSpan;
										} while(iK1 < iK2);
										pRe[size_t(iKK)] = dAK;
										pIm[size_t(iKK)] = dBK;
										iK1 = iKK;
										iK2 = iKK + iISpan;
										iJ = 1;
										do
										{
											iK1 += iKSpan;
											iK2 -= iKSpan;
											iJJ = iJ;
											dAK = dAA;
											dBK = dBB;
											dAJ = 0.0;
											dBJ = 0.0;
											iK = 1;
											do
											{
												iK++;
												dAK += pRTmp[size_t(iK - 1)] * pCos[size_t(iJJ - 1)];
												dBK += pITmp[size_t(iK - 1)] * pCos[size_t(iJJ - 1)];
												iK++;
												dAJ += pRTmp[size_t(iK - 1)] * pSin[size_t(iJJ - 1)];
												dBJ += pITmp[size_t(iK - 1)] * pSin[size_t(iJJ - 1)];
												iJJ += iJ;
												if(iJJ > iJF)
												{
													iJJ -= iJF;
												}
											} while(iK < iJF);
											iK = iJF - iJ;
											pRe[size_t(iK1)] = dAK - dBJ;
											pIm[size_t(iK1)] = dBK + dAJ;
											pRe[size_t(iK2)] = dAK + dBJ;
											pIm[size_t(iK2)] = dBK - dAJ;
											iJ++;
										} while(iJ < iK);
										iKK += iISpan;
									} while(iKK <= iNN);
									iKK -= iNN;
								} while(iKK <= iKSpan);
								break;
							}
							// Multiply by rotation factor (except for factors of 2 and 4)
							if(iI == iMFactor)
							{
								bGotoPermuteResultsLabel = true;
							}
							else
							{
								iKK = iJC + 1;
								do
								{
									dC2 = 1.0 - dCD;
									dS1 = dSD;
									do
									{
										dC1 = dC2;
										dS2 = dS1;
										iKK += iKSpan;
										do
										{
											do
											{
												dAK = pRe[size_t(iKK)];
												pRe[size_t(iKK)] = dC2 * dAK - dS2 * pIm[size_t(iKK)];
												pIm[size_t(iKK)] = dS2 * dAK + dC2 * pIm[size_t(iKK)];
												iKK += iISpan;
											} while(iKK <= iNT);
											dAK = dS1 * dS2;
											dS2 = dS1 * dC2 + dC1 * dS2;
											dC2 = dC1 * dC2 - dAK;
											iKK = iKK - iNT + iKSpan;
										} while(iKK <= iISpan);
										dC2 = dC1 - (dCD * dC1 + dSD * dS1);
										dS1 += dSD * dC1 - dCD * dS1;
										dC1 = 2.0 - (dC2 * dC2 + dS1 * dS1);
										dS1 *= dC1;
										dC2 *= dC1;
										iKK = iKK - iISpan + iJC;
									} while(iKK <= iKSpan);
									iKK = iKK - iKSpan + iJC + iInc;
								} while(iKK <= iJC + iJC);
							}
							break;
						}
						// bGotoPermuteResultsLabel = true;
					} while(!bGotoPermuteResultsLabel); // End of loop
					// Permute the results to normal order - done in two stages
					// Permute for square factors of n
					// ------ PERMUTE_RESULTS_LABEL ------
					pPerm[0] = iNS;
					if(iKT != 0)
					{
						iK = iKT + iKT + 1;
						if(iMFactor < iK)
						{
							iK--;
						}
						iJ = 1;
						pPerm[size_t(iK)] = iJC;
						do
						{
							pPerm[size_t(iJ)] = pPerm[size_t(iJ - 1)] / fft_factor(pFactor, size_t(iJ - 1));
							pPerm[size_t(iK - 1)] = pPerm[size_t(iK)] * fft_factor(pFactor, size_t(iJ - 1));
							iJ++;
							iK--;
						} while(iJ < iK);
						iK3 = pPerm[size_t(iK)];
						iKSpan = pPerm[1];
						iKK = iJC + 1;
						iK2 = iKSpan + 1;
						iJ = 1;
						if(ulNPass != ulNTotal)
						{	// Permutation for multivariate transform
							// ------ PERMUTE_MULTI_LABEL ------
							do
							{
								bGotoPermuteMultiLabel = false;
								do
								{
									do
									{
										iK = iKK + iJC;
										do
										{	// swap pRe[iKK] <> pRe[iK2], pIm[iKK] <> pIm[iK2]
											dAK = pRe[size_t(iKK)];
											pRe[size_t(iKK)] = pRe[size_t(iK2)];
											pRe[size_t(iK2)] = dAK;
											dBK = pIm[size_t(iKK)];
											pIm[size_t(iKK)] = pIm[size_t(iK2)];
											pIm[size_t(iK2)] = dBK;
											iKK += iInc;
											iK2 += iInc;
										} while(iKK < iK);
										iKK += iNS - iJC;
										iK2 += iNS - iJC;
									} while(iKK < iNT);
									iK2 = iK2 - iNT + iKSpan;
									iKK = iKK - iNT + iJC;
								} while(iK2 < iNS);
								do
								{
									do
									{
										iK2 -= pPerm[size_t(iJ - 1)];
										iJ++;
										iK2 = pPerm[size_t(iJ)] + iK2;
									} while(iK2 > pPerm[size_t(iJ - 1)]);
									iJ = 1;
									do
									{
										if(iKK < iK2)
										{	// Goto Permute_Multi_Label
											bGotoPermuteMultiLabel = true;
										}
										else
										{
											iKK += iJC;
											iK2 += iKSpan;
										}
									} while(iK2 < iNS && !bGotoPermuteMultiLabel);
								} while(iKK < iNS && !bGotoPermuteMultiLabel);
							} while(bGotoPermuteMultiLabel);
						}	// end of permutation for multivariate transform
						else
						{	// Permutation for single-variate transform (optional code)
							// ------ PERMUTE_SINGLE_LABEL ------
							do
							{
								bGotoPermuteSingleLabel = false;
								do
								{	// swap pRe[iKK] <> pRe[iK2], pIm[iKK] <> pIm[iK2]
									dAK = pRe[size_t(iKK)];
									pRe[size_t(iKK)] = pRe[size_t(iK2)];
									pRe[size_t(iK2)] = dAK;
									dBK = pIm[size_t(iKK)];
									pIm[size_t(iKK)] = pIm[size_t(iK2)];
									pIm[size_t(iK2)] = dBK;
									iKK += iInc;
									iK2 += iKSpan;
								} while(iK2 < iNS);
								do
								{
									do
									{
										iK2 -= pPerm[size_t(iJ - 1)];
										iJ++;
										iK2 = pPerm[size_t(iJ)] + iK2;
									} while(iK2 > pPerm[size_t(iJ - 1)]);
									iJ = 1;
									do
									{
										if(iKK < iK2)
										{	// Goto Permute_Single_Label
											bGotoPermuteSingleLabel = true;
										}
										else
										{
											iKK += iInc;
											iK2 += iKSpan;
										}
									} while(iK2 < iNS && !bGotoPermuteSingleLabel);
								} while(iKK < iNS && !bGotoPermuteSingleLabel);
							} while(bGotoPermuteSingleLabel);
						}
						iJC = iK3;
					}
					if((iKT << 1) + 1 < iMFactor)
					{
						iISpan = pPerm[size_t(iKT)];
						// Permutation for square-free factors of n
						iJ = iMFactor - iKT;
						fft_factor(pFactor, size_t(iJ)) = 1;
						do
						{
							fft_factor(pFactor, size_t(iJ - 1)) *= fft_factor(pFactor, size_t(iJ));
							iJ--;
						} while(iJ != iKT);
						iKT++;
						iNN = fft_factor(pFactor, size_t(iKT - 1)) - 1;
						if(iNN > iMaxPerm)
						{
							RaiseException("number of factors exceeded", __LINE__, "fft_radix(double *, double *, size_t, size_t, size_t, int, int, int, size_t, size_t, double **, int *&, int *&)", __FILE__, e_DomainError);
						}
						iJ = iJJ = 0;
						while(!bLoopBreak)
						{
							iK = iKT + 1;
							iK2 = fft_factor(pFactor, size_t(iKT - 1));
							iKK = fft_factor(pFactor, size_t(iK - 1));
							iJ++;
							if(iJ > iNN)
							{
								bLoopBreak = true;
							}
							else
							{
								iJJ += iKK;
								while(iJJ >= iK2)
								{
									iJJ -= iK2;
									iK2 = iKK;
									iK++;
									iKK = fft_factor(pFactor, size_t(iK - 1));
									iJJ += iKK;
								}
								pPerm[size_t(iJ - 1)] = iJJ;
							}
						}
						// Determine the permutation cycles of length greater than 1
						iJ = 0;
						bLoopBreak = false;
						while(!bLoopBreak)
						{
							do
							{
								iJ++;
								iKK = pPerm[size_t(iJ - 1)];
							} while(iKK < 0);
							if(iKK != iJ)
							{
								do
								{
									iK = iKK;
									iKK = pPerm[size_t(iK - 1)];
									pPerm[size_t(iK - 1)] = -iKK;
								} while(iKK != iJ);
								iK3 = iKK;
							}
							else
							{
								pPerm[size_t(iJ - 1)] = -iJ;
								if(iJ == iNN)
								{
									bLoopBreak = true;
								}
							}
						}
						iMaxFactors *= iInc;
						// Reorder a and b, following the permutation cycles
						bLoopBreak = false;
						while(!bLoopBreak)
						{
							iJ = iK3 + 1;
							iNT -= iISpan;
							iI = iNT - iInc + 1;
							if(iNT < 0)
							{
								bLoopBreak = true;
							}
							else
							{
								do
								{
									do
									{
										iJ--;
									} while(pPerm[size_t(iJ - 1)] < 0);
									iJJ = iJC;
									do
									{
										iKSpan = iJJ;
										if(iJJ > iMaxFactors)
										{
											iKSpan = iMaxFactors;
										}
										iJJ -= iKSpan;
										iK = pPerm[size_t(iJ - 1)];
										iKK = iJC * iK + iI + iJ;
										iK1 = iKK + iKSpan;
										iK2 = 0;
										do
										{
											iK2++;
											pRTmp[size_t(iK2 - 1)] = pRe[iK1];
											pITmp[size_t(iK2 - 1)] = pIm[iK1];
											iK1 -= iInc;
										} while(iK1 != iKK);
										do
										{
											iK1 = iKK + iKSpan;
											iK2 = iK1 - iJC * (iK + pPerm[size_t(iK - 1)]);
											iK = -pPerm[size_t(iK - 1)];
											do
											{
												pRe[size_t(iK1)] = pRe[size_t(iK2)];
												pIm[size_t(iK1)] = pIm[size_t(iK2)];
												iK1 -= iInc;
												iK2 -= iInc;
											} while(iK1 != iKK);
											iKK = iK2;
										} while(iK != iJ);
										iK1 = iKK + iKSpan;
										iK2 = 0;
										do
										{
											iK2++;
											pRe[size_t(iK1)] = pRTmp[size_t(iK2 - 1)];
											pIm[size_t(iK1)] = pITmp[size_t(iK2 - 1)];
											iK1 -= iInc;
										} while(iK1 != iKK);
									} while(iJJ != 0);
								} while(iJ != 1);
							}
						}
					}
				}
			}
			return 0;
		}
	}

	// Function to convert all linear integers to a periodic boundary condition
	// Bind the input iIn on the circular interval between iA and iB inclusive
	int PeriodicConvert(int iIn, int iA, int iB) throw()
	{
		int iOut, iMax, iMin;
		iOut = iIn;
		iMax = iA > iB ? iA : iB;		// Calculate the minimum value
		iMin = iA < iB ? iA : iB;		// Calculate the maximum value
		if(iMax != iMin)
		{
			if(iIn > iMax)
			{	// iIn is greater than the maximum range 
				iOut = iMin + ((iIn - iMax - 1) % (iMax - iMin + 1));
			}
			else if(iIn < iMin)
			{	// iIn is less than the minimum range
				iOut = iMax - ((iMin - iIn - 1) % (iMax - iMin + 1));
			}
		}
		else
		{	// Maximum value and minimum value are the same value
			iOut = iA;
		}
		return iOut;
	}

	// Integer power function
	int PowI(int iIn, int iPow) throw(Exception)
	{
		int iOut, iIter;
		iIter = iOut = 0;
		if(iPow < 0)
		{	// Raise to the power of a negative number
			if(iIn == 0)
			{	// 0 ^ x = NaN when x < 0
				StdExtensions::RaiseException("unable to raise zero to the power of a negative number", __LINE__, "StdExtensions::PowI(int, int)", __FILE__, StdExtensions::e_DomainError);
			}
			else if(iIn == -1)
			{	// -1 ^ x = -1 when x < 0
				iOut = -1;
			}
		}
		else if(iPow > 0)
		{
			if(iIn == 0)
			{	// 0 ^ x = 0 when x > 0
				iOut = 0;
			}
			else
			{	// x ^ y = x ^ y
				iOut = 1;
				for(iIter = 0; iIter < iPow; ++iIter)
				{
					iOut *= iIn;
				}
			}
		}
		else
		{	// Raise to the power of zero
			iOut = 1;
		}
		return iOut;
	}

	// Special error functions.  Algorithms use the translated
	// code of Cody (1993).  This code is released under the
	// GPL
	// Function to return the numerical evaluation of the error function
	double Erf(double dIn) throw()
	{
		return CalErf(dIn, 0);
	}

	// Special error functions.  Algorithms use the translated
	// code of Cody (1993).  This code is released under the
	// GPL
	// Function to return the numerical evaluation of the complement error function
	double Erfc(double dIn) throw()
	{
		return CalErf(dIn, 1);
	}

	// Perform a fourier transform on a set of complex data.
	// Algorithm uses the translated code of Singleton (1968)
	// revised in 1995 by Olesen.  This code is released under
	// the GPL.
	void FFT(int iNDim, const int *pDims, double *pRe, double *pIm, bool bForward) throw(Exception)
	{
		double *pTmp[4];			// Array of pointers to hold temporary values
		int *pPerm, *pFacTemp;		// Permutation vector
		int pFactor[ulNFactor];
		// Parameters for memory management
		size_t ulSpaceAlloced, ulMaxPermAlloced;
		size_t ulNSpan, ulNPass, ulNTotal;
		int iRet, iI, iMaxFactors, iMaxPerm, iSign;
		double dScaling = 0.0;
		ulSpaceAlloced = ulMaxPermAlloced = ulNSpan = ulNPass = ulNTotal = 0;
		iRet = iI = iMaxFactors = iMaxPerm = iSign = 0;
		pPerm = NULL;
		pFacTemp = &(pFactor[0]);
		pTmp[0] = pTmp[1] = pTmp[2] = pTmp[3] = NULL;
		iSign = bForward ? 1 : -1;
		dScaling = bForward ? 1.0 : -1.0;
		if(iNDim <= 0)
		{	// Invalid number of dimensions entered
			RaiseException("invalid input for dimension number", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_DomainError);
		}
		else if(pDims == NULL)
		{	// Invalid dimension array entered
			RaiseException("invalid input for dimension extent array", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_DomainError);
		}
		else if(pRe == NULL)
		{	// Invalid real component entered
			RaiseException("invalid input for data real component", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_DomainError);
		}
		else if(pIm == NULL)
		{	// Invalid imaginary component entered
			RaiseException("invalid input for data imaginary component", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_DomainError);
		}
		else
		{	// Input parameters are fine - continue with calculation
			try
			{	// Tally the number of elements in the data array and determine
				// the number of dimensions
				ulNTotal = 1;
				for(iI = 0; iI < iNDim; ++iI)
				{
					if(pDims[size_t(iI)] <= 0)
					{
						RaiseException("invalid dimension extent entry", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_DomainError);
					}
					ulNTotal *= size_t(pDims[size_t(iI)]);
				}
				// Determine maximum number of factors and permutations
				// Follow John Beale's example, just use the largest dimension and don't
				// worry about excess allocation.  May be someone else will do it?
				iMaxFactors = iMaxPerm = 1;
				for(iI = 0; iI < iNDim; ++iI)
				{
					ulNSpan = size_t(pDims[size_t(iI)]);
					if(ulNSpan > size_t(iMaxFactors))
					{
						iMaxFactors = int(ulNSpan);
					}
					if(ulNSpan > size_t(iMaxPerm))
					{
						iMaxPerm = int(ulNSpan);
					}
				}
				// Loop over the dimensions
				ulNSpan = 1;
				for(iI = 0; iI < iNDim && iRet == 0; ++iI)
				{
					ulNPass = size_t(pDims[size_t(iI)]);
					ulNSpan *= ulNPass;
					// Call Singleton's mixed radix routine
					iRet = fft_radix(pRe, pIm, ulNTotal, ulNPass, ulNSpan, iSign, iMaxFactors, iMaxPerm, ulSpaceAlloced, ulMaxPermAlloced, pTmp, pPerm, pFacTemp);
				}
				// Continue if Singleton's mixed radix routine exited OK
				if(iRet == 0)
				{	// Divide through by normalising constant
					if(dScaling != 0.0 && dScaling != 1.0)
					{
						if(iSign < 0)
						{
							iSign = -iSign;
						}
						if(dScaling < 0.0)
						{
							dScaling = (dScaling < -1.0) ? sqrt(double(ulNTotal)) : double(ulNTotal);
						}
						// Multiplication is often faster than division
						dScaling = 1.0 / dScaling;
						for(iI = 0; iI < int(ulNTotal); iI += iSign)
						{
							pRe[size_t(iI)] *= dScaling;
							pIm[size_t(iI)] *= dScaling;
						}
					}
				}
				else
				{
					RaiseException("abnormal termination of Singleton's mixed radix routine", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_UnknownError);
				}
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - delete temporary values before rethrowing exception
				fft_free(ulSpaceAlloced, ulMaxPermAlloced, pTmp, pPerm);
				throw;
			}
			catch(...)
			{	// Non standard exception thrown - delete temporary values before throwing standard exception
				fft_free(ulSpaceAlloced, ulMaxPermAlloced, pTmp, pPerm);
				RaiseException("unknown error thrown during fast Fourier transform", __LINE__, "StdExtensions::FFT(int, const int *, double *, double *, bool)", __FILE__, e_UnknownError);
			}
			// Delete temporary values
			fft_free(ulSpaceAlloced, ulMaxPermAlloced, pTmp, pPerm);
		}
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
