#ifndef __MATHFUNCS_H__
#define __MATHFUNCS_H__

#include <cmath>
#include <cfloat>
#include "Exception.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{
	// Mathematical constants
	const double EXT_PI = 3.1415926535897932384626433832795028841968;		// Pi

	// Function to convert all linear integers to a periodic boundary condition
	int PeriodicConvert(int iIn, int iA, int iB) throw();

	// Integer power function
	int PowI(int iIn, int iPow) throw(Exception);

	// Return the maximum and minimum values
	inline int Max(int iA, int iB) throw() {return iA > iB ? iA : iB;}
	inline int Min(int iA, int iB) throw() {return iA < iB ? iA : iB;}
	inline double Max(double dA, double dB) throw() {return dA > dB ? dA : dB;}
	inline double Min(double dA, double dB) throw() {return dA < dB ? dA : dB;}

	// Functions to square a number
	inline int Sqrd(int iIn) throw() {return iIn * iIn;}
	inline double Sqrd(double dIn) throw() {return dIn * dIn;}

	// Functions to truncate a double
	inline double Trunc(double dIn) throw() {return double(int(dIn));}

	// Special error functions.  Algorithms use the translated
	// code of Cody (1993).  This code is released under the
	// GPL
	// Function to return the numerical evaluation of the error function
	double Erf(double dIn) throw();
	// Function to return the numerical evaluation of the complement error function
	double Erfc(double dIn) throw();

	// Perform a fourier transform on a set of complex data.
	// Algorithm uses the translated code of Singleton (1968)
	// revised in 1995 by Olesen.  This code is released under
	// the GPL.
	void FFT(int iNDim, const int *pDims, double *pRe, double *pIm, bool bForward = true) throw(Exception);
	inline void InvFFT(int iNDim, const int *pDims, double *pRe, double *pIm) throw(StdExtensions::Exception) {FFT(iNDim, pDims, pRe, pIm, false);}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
