#ifndef __SPECTRALSYNTHESIS_H__
#define __SPECTRALSYNTHESIS_H__

#include "RInterface.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

// Function to generate landscapes according to fractional Brownian motion
// this funciton uses an updated version of the algorithm published by
// Keitt (2000) allowing for conjugate symmetry conditions to be satisfied
// in all dimensions according to Peitgen and Saupe (1988)
//
//		DimExtents	-	Integer vector describing the length of each dimension
//		Hursts		-	Double vector describing the Hurst exponent in each dimension
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP SpectralSynthesis(SEXP pArgs);
#else
	extern "C" SEXP SpectralSynthesis(SEXP pArgs);
#endif

// Function to generate landscapes according to hierarchical curdling algorithms
// of O'Neill, Gardner and Turner (1992) and Lavorel, Gardner and O'Neill (1993)
//
//		DimExtents	-	Integer vector describing the length of each dimension
//		HierBoxes	-	A matrix describing the size of the tiles at each hierarchical
//						subdivision.  The first dimension of the matrix is the hierarchy
//						of the tile and the second dimension of matrix describes the
//						size of the tile in each dimension of the output
//		HierProp	-	Integer vector describing the number of tiles to keep
//						at each hierarchical sweep
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP HierarchicalCurdling(SEXP pArgs);
#else
	extern "C" SEXP HierarchicalCurdling(SEXP pArgs);
#endif

// Function to generate a simple percolation map according to the methods described
// in Gardner et al. (1987)
//
//		DimExtents	-	Integer vector describing the length of each dimension
//		Props		-	Integer vector of proportions of the land to designate to
//						each habitat type
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP PercolationMap(SEXP pArgs);
#else
	extern "C" SEXP PercolationMap(SEXP pArgs);
#endif


//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
